/* ----------------------------------------------------------------------------

 Question Server
 GitLab: <https://gitlab.com/MrFry/mrfrys-node-server>

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.

 ------------------------------------------------------------------------- */

// package requires
import express from 'express'
import bodyParser from 'body-parser'
import busboy from 'connect-busboy'
import fs from 'fs'
const app = express()

// other requires
import logger from '../../utils/logger'
import { SetupData } from '../../server'
import { ModuleType, Request } from '../../types/basicTypes'

// stuff gotten from server.js
let publicdirs = []
let url = ''

function GetApp(): ModuleType {
  const publicDir = publicdirs[0]
  if (!publicDir) {
    throw new Error(`No public dir! ( Stuff )`)
  }

  // files in public dirs
  const listedFiles = './' + publicDir + 'files'

  app.set('view engine', 'ejs')
  app.set('views', ['./src/modules/stuff/views', './src/sharedViews'])
  publicdirs.forEach((pdir) => {
    logger.Log(`Using public dir: ${pdir}`)
    app.use(express.static(pdir))
  })
  app.use(
    busboy({
      limits: {
        fileSize: 10000 * 1024 * 1024,
      },
    })
  )
  app.use(bodyParser.json())
  app.use(
    bodyParser.urlencoded({
      limit: '5mb',
      extended: true,
    })
  )
  app.use(
    bodyParser.json({
      limit: '5mb',
    })
  )

  // --------------------------------------------------------------

  // app, '/*.mp4', 'video/mp4', 'stuff/video'
  function appGetFileType(app, wildcard, contentType, pageToRender) {
    app.get(wildcard, function(req, res) {
      const path = decodeURIComponent(req.url)
      let fp: any = path
      if (path.includes('?')) {
        fp = path.split('?')
        fp.pop()
        fp = fp.join('/')
      }
      const fpath = listedFiles + fp
      if (!fs.existsSync(fpath)) {
        res.render('nofile', {
          missingFile: fp,
          url,
        })
        return
      }
      if (req.query.stream || !pageToRender) {
        const stat = fs.statSync(fpath)
        const fileSize = stat.size
        const range = req.headers.range
        if (range) {
          const parts = range.replace(/bytes=/, '').split('-')
          const start = parseInt(parts[0], 10)
          const end = parts[1] ? parseInt(parts[1], 10) : fileSize - 1
          const chunksize = end - start + 1
          const file = fs.createReadStream(fpath, { start, end })
          const head = {
            'Content-Range': `bytes ${start}-${end}/${fileSize}`,
            'Accept-Ranges': 'bytes',
            'Content-Length': chunksize,
            'Content-Type': contentType,
          }
          res.writeHead(206, head)
          file.pipe(res)
        } else {
          const head = {
            'Content-Length': fileSize,
            'Content-Type': contentType,
          }
          res.writeHead(200, head)
          fs.createReadStream(fpath).pipe(res)
        }
      } else {
        logger.LogReq(req)
        const fname = fpath.split('/').pop()
        res.render(pageToRender, {
          path: fp,
          fname,
          url,
          contentType,
          albumArt: GetAlbumArt(path),
        })
      }
    })
  }

  const fileTypes = [
    ['/*.mp4', 'video/mp4', 'video'],
    ['/*.mkv', 'audio/x-matroska', 'video'],
    ['/*.mp3', 'audio/mpeg', 'audio'],
    ['/*.pdf', 'application/pdf'],
    ['/*.zip', 'application/zip'],
  ]

  function GetAlbumArt(path) {
    let tmp = path.split('.')
    tmp.pop()
    tmp = tmp.join('.').split('/')
    const last = tmp.pop()

    return tmp.join('/') + '/.' + last + '.png'
  }

  fileTypes.forEach((t) => {
    appGetFileType(app, t[0], t[1], t[2])
  })

  app.get('/*', function(req: Request, res) {
    const parsedUrl = decodeURIComponent(req.url)
    let curr =
      listedFiles +
      '/' +
      parsedUrl.substring('/'.length, parsedUrl.length).split('?')[0]
    let relPath = curr.substring(listedFiles.length, curr.length)

    if (relPath[relPath.length - 1] !== '/') {
      relPath += '/'
    }

    const temp = relPath.split('/')
    let prevDir = ''
    for (let i = 0; i < temp.length - 2; i++) {
      prevDir += temp[i] + '/'
    }

    // curr = curr.replace(/\//g, "/");
    // relPath = relPath.replace(/\//g, "/");

    logger.LogReq(req)

    try {
      const stat = fs.lstatSync(curr)
      if (stat.isDirectory() || stat.isSymbolicLink()) {
        if (curr[curr.length - 1] !== '/') {
          curr += '/'
        }

        const folders = []

        const files = fs.readdirSync(curr)
        files.sort(function(a, b) {
          return (
            fs.statSync(curr + b).mtime.getTime() -
            fs.statSync(curr + a).mtime.getTime()
          )
        })

        files.forEach((item) => {
          if (item[0] !== '.') {
            const res: any = { name: item }
            const stat = fs.statSync(curr + '/' + item)
            const fileSizeInBytes = stat['size']
            res.size = Math.round(fileSizeInBytes / 1000000)

            res.path = relPath
            if (res.path[res.path.length - 1] !== '/') {
              res.path += '/'
            }
            res.path += item

            res.mtime = stat['mtime'].toLocaleString()
            res.isDir = stat.isDirectory()

            folders.push(res)
          }
        })

        res.render('folders', {
          folders: folders,
          dirname: relPath,
          prevDir,
          url,
        })
      } else {
        const fileStream = fs.createReadStream(curr)
        fileStream.pipe(res)
      }
    } catch (err) {
      res.render('nofile', {
        missingFile: relPath,
        url,
      })
    }
  })

  // -----------------------------------------------------------------------------------------------

  app.get('*', function(req: Request, res) {
    res.status(404).render('404')
  })

  app.post('*', function(req: Request, res) {
    res.status(404).render('404')
  })

  return {
    app: app,
  }
}

export default {
  name: 'Stuff',
  getApp: GetApp,
  setup: (data: SetupData): void => {
    url = data.url
    publicdirs = data.publicdirs
  },
}
